/*
 * FILE --- erfinv.hh
 *
 * PURPOSE --- Source file for a function that implements the inverse of
 * the function erf.
 *
 */

// Include Files
#include "erfinv.hh"

// Define this macro if you want to use this utility without linking
// to the RCS Library
#ifdef NO_RCS
#include <stdio.h>            /* printf function */
#else
#include "rcs_prnt.hh"          /* rcs_print_ functions */
#endif

double erfinv(double y)
{
  double x = 0.0; // The output
  double z = 0.0; // Intermadiate variable
  double y0 = 0.7; // Central range variable

  // Coefficients in rational approximations.
  double a[4] = { 0.886226899, -1.645349621,  0.914624893, -0.140543331};
  double b[4] = {-2.118377725,  1.442710462, -0.329097515,  0.012229801};  
  double c[4] = {-1.970840454, -1.624906493,  3.429567803,  1.641345311};
  double d[2] = { 3.543889200,  1.637067800};

  if ( (y < -1.0) || (1.0 < y) )
    {
#ifdef NO_RCS
      printf("erfinv: The value out of the range of the function");
#else
      rcs_print("erfinv: The value out of the range of the function");
#endif
      x = log(-1.0);
    }
  else if ( (y == -1.0) || (1.0 == y) )
    {
      x = - y*log(0.0);
    }
  else if ( (-1.0 < y) && (y < -y0) )
    {
      z = sqrt(-log((1.0+y)/2.0));
      x = -(((c[3]*z+c[2])*z+c[1])*z+c[0])/((d[1]*z+d[0])*z+1.0);
    }
  else
    {
      if ( (-y0 < y) && (y < y0) )
	{
	  z = y*y;
	  x = y*(((a[3]*z+a[2])*z+a[1])*z+a[0])/
	    ((((b[3]*z+b[3])*z+b[1])*z+b[0])*z+1.0);
	}
      else if ( (y0 < y) && (y < 1.0) )
	{
	  z = sqrt(-log((1.0-y)/2.0));
	  x = (((c[3]*z+c[2])*z+c[1])*z+c[0])/((d[1]*z+d[0])*z+1.0);
	}
      
      // Two steps of Newton-Raphson correction to full accuracy.
      x = x - (erf(x) - y) / (2.0/sqrt(PI) * exp(-x*x));
      x = x - (erf(x) - y) / (2.0/sqrt(PI) * exp(-x*x));
    }
  return(x);
}
 
